/*
  * Copyright 2012  Samsung Electronics Co., Ltd
  *
  * Licensed under the Flora License, Version 1.1 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *     http://floralicense.org/license/
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */

#include "exceptionsview.h"

#include <vector>
#include <string>
#include <cstring>
#include <Elementary.h>

#include <dpl/foreach.h>
#include <dpl/assert.h>
#include <dpl/string.h>
#include <dpl/wrt-dao-ro/widget_dao_read_only.h>
#include <wrt-commons/security-origin-dao/security_origin_dao.h>

//#include <ui-gadget.h>
#include "util.h"

namespace WebAppDetailSetting {

namespace  {
// ELM
const char* const ELM_PART_TEXT_1 = "elm.text.1";
const char* const ELM_PART_TEXT_2 = "elm.text.2";
const char* const ELM_PART_TEXT_3 = "elm.text.3";
const char* const ELM_PART_DEFAULT = "default";
const char* const ELM_PART_BUTTON1 = "button1";
const char* const ELM_PART_BUTTON2 = "button2";
const char* const ELM_WIDGET_TYPE_POPUP = "elm_popup";
const char* const ELM_GENLIST_ITEM_CLASS_STYLE_DEFAULT = "default";
const char* const ELM_GENLIST_ITEM_CLASS_STYLE_3TEXT_1ICON = "3text.1icon";
const char* const EVAS_SMART_CALLBACK_CLICKED = "clicked";
// TEXT
std::vector<std::string> featureText =
    {"Geolocation", "Web Notification", "User media", "Fullscreen", "Web Database"};
std::vector<std::string> resultText = {"Allow", "Deny"};
const unsigned int INDEX_TEXT_ALLOW = 0;
const unsigned int INDEX_TEXT_DENY = 1;
const char* const SCHEME_FILE = "file";
const char* const TEXT_LOCAL = "local";
const char* const TEXT_EMPTY = "No exceptions data";
const char* const TEXT_POPUP_CONTENT = "Do you want to remove data?";
const char* const TEXT_POPUP_L_BUTTON = "YES";
const char* const TEXT_POPUP_R_BUTTON = "NO";
const char* const TEXT_NAVIFRAME_TITLE = "Allow list";

}

ExceptionsView::ExceptionsView(Evas_Object* navi, const std::string& pkgId) :
    m_naviframe(navi)
{
    m_securityOriginDao =
        SecurityOriginDB::SecurityOriginDAOPtr(
            new SecurityOriginDB::SecurityOriginDAO(
                DPL::FromASCIIString(pkgId)));
}

ExceptionsView::~ExceptionsView(void)
{
}

bool ExceptionsView::loadView(void)
{
    using namespace SecurityOriginDB;

    Evas_Object* genlist = NULL;
    genlist = elm_genlist_add(m_naviframe);

    SecurityOriginDB::SecurityOriginDataList securityOriginDataList =
        m_securityOriginDao->getSecurityOriginDataList();

    if (securityOriginDataList.empty()) {
        loadEmptyView(genlist);
    } else {
        loadExceptionsSettingsView(genlist, securityOriginDataList);
    }

    elm_naviframe_item_push(m_naviframe,
                            TEXT_NAVIFRAME_TITLE,
                            NULL,
                            NULL,
                            genlist,
                            NULL);
    return true;
}

void ExceptionsView::loadEmptyView(Evas_Object* genlist)
{
    using namespace SecurityOriginDB;

    Assert(genlist);
    elm_genlist_block_count_set(genlist, 1);

    // create Security Settings menu
    static Elm_Genlist_Item_Class itcData = {0,};
    itcData.item_style = ELM_GENLIST_ITEM_CLASS_STYLE_DEFAULT;
    itcData.func.text_get = getEmptyTextCallback;
    itcData.func.content_get = NULL;
    itcData.func.state_get = NULL;
    itcData.func.del = NULL;

    Elm_Object_Item* emptyItem =
        elm_genlist_item_append(genlist,
                                &itcData,
                                NULL,
                                NULL,
                                ELM_GENLIST_ITEM_NONE,
                                NULL,
                                NULL);
    elm_genlist_item_select_mode_set(emptyItem,
                                     ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
}

void ExceptionsView::loadExceptionsSettingsView(
    Evas_Object* genlist,
    SecurityOriginDB::SecurityOriginDataList securityOriginDataList)
{
    using namespace SecurityOriginDB;

    Assert(genlist);
    elm_genlist_block_count_set(genlist, securityOriginDataList.size());
    // create Security Settings menu
    static Elm_Genlist_Item_Class itcExceptionsSettings = {0,};
    FOREACH(it, securityOriginDataList) {
        itcExceptionsSettings.item_style =
            ELM_GENLIST_ITEM_CLASS_STYLE_3TEXT_1ICON;
        itcExceptionsSettings.func.text_get = getTextCallback;
        itcExceptionsSettings.func.content_get = NULL;
        itcExceptionsSettings.func.state_get = NULL;
        itcExceptionsSettings.func.del = delCallback;

        ExceptionsSettingsData* data =
            new ExceptionsSettingsData(m_securityOriginDao.get(),
                                       *(*it));
        Elm_Object_Item *objectItem;
        objectItem = elm_genlist_item_append(genlist,
                                             &itcExceptionsSettings,
                                             static_cast<void*>(data),
                                             NULL,
                                             ELM_GENLIST_ITEM_NONE,
                                             selectedCallback,
                                             NULL);
        if (m_securityOriginDao->isReadOnly(*(*it))) {
            elm_object_item_disabled_set(objectItem, EINA_TRUE);
        }
    }
}

char* ExceptionsView::getEmptyTextCallback(
    void* /*data*/,
    Evas_Object* /*obj*/,
    const char* /*part*/)
{
    return strdup(TEXT_EMPTY);
}

char* ExceptionsView::getTextCallback(
    void* data,
    Evas_Object* obj,
    const char* part)
{
    using namespace SecurityOriginDB;
    Assert(data);
    ExceptionsSettingsData* settingsData =
        static_cast<ExceptionsSettingsData*>(data);
    Assert(part);
    std::string partStr = part;

    if (partStr == ELM_PART_TEXT_1) {
        // display feature text
        return strdup(
            featureText[settingsData->m_originData.feature].c_str());
    } else if (partStr == ELM_PART_TEXT_2) {
        // display host text
        if (DPL::ToUTF8String(settingsData->m_originData.origin.scheme) ==
                SCHEME_FILE)
        {
            // in case of file scheme, display "local"
            return strdup(TEXT_LOCAL);
        } else {
            return strdup(
                DPL::ToUTF8String(
                    settingsData->m_originData.origin.host).c_str());
        }
    } else if (partStr == ELM_PART_TEXT_3) {
        // display result text
        Result result =
            settingsData->m_originDao->getResult(
                settingsData->m_originData);
        if (result == RESULT_ALLOW_ALWAYS) {
            return strdup(resultText[INDEX_TEXT_ALLOW].c_str());
        } else if (result == RESULT_DENY_ALWAYS) {
            return strdup(resultText[INDEX_TEXT_DENY].c_str());
        } else {
            return NULL;
        }
    }

    return NULL;
}

void ExceptionsView::delCallback(void* data, Evas_Object* /*obj*/)
{
    Assert(data);
    ExceptionsSettingsData* exceptionsSettingsData =
        static_cast<ExceptionsSettingsData*>(data);
    delete exceptionsSettingsData;
}

void ExceptionsView::selectedCallback(
    void* data,
    Evas_Object* obj,
    void* eventInfo)
{
    Assert(eventInfo);
    Elm_Object_Item* item = static_cast<Elm_Object_Item*>(eventInfo);
    elm_genlist_item_selected_set(item, EINA_FALSE);

    // create popup
    Evas_Object* topObject = elm_object_top_widget_get(obj);

    Evas_Object* popup = elm_popup_add(topObject);
    if (!popup) {
        return;
    }
    evas_object_size_hint_align_set(popup, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(popup,
                                     EVAS_HINT_EXPAND,
                                     EVAS_HINT_EXPAND);
    elm_object_part_text_set(popup, ELM_PART_DEFAULT, TEXT_POPUP_CONTENT);

    Evas_Object* lButton = elm_button_add(popup);
    elm_object_text_set(lButton, TEXT_POPUP_L_BUTTON);
    evas_object_smart_callback_add(lButton,
                                   EVAS_SMART_CALLBACK_CLICKED,
                                   removeItemCallback,
                                   static_cast<void*>(item));
    elm_object_part_content_set(popup, ELM_PART_BUTTON1, lButton);
    evas_object_show(lButton);

    Evas_Object* rButton = elm_button_add(popup);
    elm_object_text_set(rButton, TEXT_POPUP_R_BUTTON);
    evas_object_smart_callback_add(rButton,
                                   EVAS_SMART_CALLBACK_CLICKED,
                                   closePopupCallback,
                                   static_cast<void*>(popup));
    elm_object_part_content_set(popup, ELM_PART_BUTTON2, rButton);
    evas_object_show(rButton);
    evas_object_show(popup);
}

void ExceptionsView::removeItemCallback(
    void* data,
    Evas_Object* obj,
    void* eventInfo)
{
    Assert(data);
    Elm_Object_Item* item = static_cast<Elm_Object_Item*>(data);
    ExceptionsSettingsData* settingsData =
        static_cast<ExceptionsSettingsData*>(elm_object_item_data_get(item));
    Assert(settingsData);

    // remove data in the security origin dao
    settingsData ->m_originDao->removeSecurityOriginData(
        settingsData->m_originData);

    // remove item in the genlist
    elm_object_item_del(item);

    // remove popup
    Evas_Object* popup = obj;
    const char* type = elm_object_widget_type_get(popup);
    while (type != NULL && strcmp(type, ELM_WIDGET_TYPE_POPUP)) {
        popup = elm_object_parent_widget_get(popup);
        type = elm_object_widget_type_get(popup);
    }
    evas_object_del(popup);
}

void ExceptionsView::closePopupCallback(
    void* data,
    Evas_Object* /*obj*/,
    void* /*eventInfo*/)
{
    Assert(data);
    evas_object_del(static_cast<Evas_Object*>(data));
}
} /* WebAppDetailSetting */
