/*
  * Copyright 2013  Samsung Electronics Co., Ltd
  *
  * Licensed under the Flora License, Version 1.1 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *     http://floralicense.org/license/
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
/**
 * @file    origin_view.cpp
 * @author  Leerang Song (leerang.song@samsung.com)
 */


#include "origin_view.h"

#include <memory>
#include <set>
#include <string>
#include <Elementary.h>
#include <dpl/assert.h>
#include <dpl/string.h>
#include <dpl/foreach.h>
#include <dpl/fast_delegate.h>
#include <wrt-commons/security-origin-dao/security_origin_dao.h>

#include "util.h"
#include "context.h"
#include "menu_data.h"
#include "permission_view.h"
#include "wrt_setting_text.h"

using namespace SecurityOriginDB;

namespace WebAppDetailSetting {

namespace {

OriginView* getOriginViewInstance(void* data)
{
    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    OriginView* This = static_cast<OriginView*>(glData->data);
    Assert(This);
    return This;
}
} // anonymous namespace

OriginView::OriginView(Evas_Object* parent, const std::string& pkgId) :
    m_genlist(NULL)
{
    // initialize context
    m_context.parent = parent;
    m_context.dao =
        SecurityOriginDAOPtr(
            new SecurityOriginDAO(DPL::FromASCIIString(pkgId)));
    m_context.cbs.originViewUpdate =
        DPL::MakeDelegate(this, &OriginView::updateCallback);
}

OriginView::~OriginView(void)
{
    m_permissionView.reset();
}

void OriginView::init(void)
{
    m_context.setOriginData();

    m_genlist = elm_genlist_add(m_context.parent);
    if (m_context.originData.empty()) {
        loadEmptyView();
    } else {
        loadView();
    }

    elm_naviframe_item_push(m_context.parent,
                            WRT_SETTING_BODY_WEBSITE_SETTINGS,
                            NULL,
                            NULL,
                            m_genlist,
                            NULL);
}

void OriginView::loadView(void)
{
    Assert(m_genlist);
    elm_genlist_block_count_set(m_genlist, m_context.originData.size());

    unsigned int index = 0;
    FOREACH(it, m_context.originData) {
        Elm_Genlist_Item_Class* itc = elm_genlist_item_class_new();
        itc->item_style = "default";
        itc->func.text_get = getTextCallback;
        itc->func.content_get = NULL;
        itc->func.state_get = NULL;
        itc->func.del = delCallback;

        GenlistData* data(new GenlistData(index++, static_cast<void*>(this)));
        elm_genlist_item_append(m_genlist,
                                itc,
                                static_cast<void*>(data),
                                NULL,
                                ELM_GENLIST_ITEM_NONE,
                                selectedCallback,
                                NULL);
        elm_genlist_item_class_free(itc);
    }
}

void OriginView::loadEmptyView(void)
{
    Assert(m_genlist);
    elm_genlist_block_count_set(m_genlist, 1);

    Elm_Genlist_Item_Class* itc = elm_genlist_item_class_new();
    itc->item_style = "default";
    itc->func.text_get = getEmptyTextCallback;
    itc->func.content_get = NULL;
    itc->func.state_get = NULL;
    itc->func.del = NULL;
    Elm_Object_Item* emptyItem =
        elm_genlist_item_append(m_genlist,
                                itc,
                                NULL,
                                NULL,
                                ELM_GENLIST_ITEM_NONE,
                                NULL,
                                NULL);
    elm_genlist_item_class_free(itc);
    elm_genlist_item_select_mode_set(emptyItem,
                                     ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
}

char* OriginView::getEmptyTextCallback(void* data,
                                  Evas_Object* obj,
                                  const char* part)
{
    UNUSED_PARAM(data);
    UNUSED_PARAM(obj);
    UNUSED_PARAM(part);

    return strdup(WRT_SETTING_BODY_EMPTY);
}

char* OriginView::getTextCallback(
    void* data,
    Evas_Object* obj,
    const char* part)
{
    UNUSED_PARAM(obj);
    UNUSED_PARAM(part);

    // get OriginView instance
    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    OriginView* This = static_cast<OriginView*>(glData->data);
    Assert(This);
    return strdup(This->m_context.originData[glData->index].c_str());
}

void OriginView::delCallback(void* data, Evas_Object* obj)
{
    UNUSED_PARAM(obj);

    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    delete glData;
}

void OriginView::selectedCallback(void* data,
                                  Evas_Object* obj,
                                  void* eventInfo)
{
    UNUSED_PARAM(data);
    UNUSED_PARAM(obj);

    // disable select
    Assert(eventInfo);
    Elm_Object_Item* item = static_cast<Elm_Object_Item*>(eventInfo);
    elm_genlist_item_selected_set(item, EINA_FALSE);

    // get OriginView instance
    GenlistData* glData =
        static_cast<GenlistData*>(elm_object_item_data_get(item));
    OriginView* This = static_cast<OriginView*>(glData->data);
    Assert(This);

    // reset old permission view
    This->m_permissionView.reset();

    // get selected origin by index
    This->m_context.setSelectedOrigin(glData->index);

    // create permission view
    This->m_permissionView.reset(new PermissionView(This->m_context));
    This->m_permissionView->init();
}

void OriginView::updateCallback(void)
{
    _D("called");
    m_context.setOriginData();
    elm_genlist_clear(m_genlist);
    if (m_context.originData.empty()) {
        loadEmptyView();
    } else {
        loadView();
    }
}
} // namespace WebAppDetailSetting
