/*
  * Copyright 2013  Samsung Electronics Co., Ltd
  *
  * Licensed under the Flora License, Version 1.1 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *     http://floralicense.org/license/
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
/**
 * @file    permission_view.cpp
 * @author  Leerang Song (leerang.song@samsung.com)
 */


#include "permission_view.h"

#include <string>
#include <Elementary.h>
#include <efl_assist.h>
#include <dpl/assert.h>
#include <dpl/foreach.h>
#include <wrt-commons/security-origin-dao/security_origin_dao.h>

#include "util.h"
#include "context.h"
#include "menu_data.h"
#include "permission_delete_view.h"
#include "ctxpopup_view.h"
#include "wrt_setting_text.h"

using namespace SecurityOriginDB;

namespace WebAppDetailSetting {
namespace {
// ETC
const char* const APP_SCHEME_ORIGIN = "local";
} // anonymous namespace

PermissionView::PermissionView(Context& context) :
    m_context(context),
    m_ctxpopupView(new CtxpopupView())
{
    // initialize context
    m_context.cbs.permissionViewUpdate =
        DPL::MakeDelegate(this, &PermissionView::updateCallback);
}

PermissionView::~PermissionView(void)
{
}

void PermissionView::init(void)
{
    // init data
    m_context.setPermissionData();

    // init genlist
    m_genlist = elm_genlist_add(m_context.parent);
    ea_object_event_callback_add(m_context.parent,
                                 EA_CALLBACK_MORE,
                                 selectedMoreCallback,
                                 this);
    if (m_context.permissionData.empty()) {
        loadEmptyView();
    } else {
        loadView();
    }
    Elm_Object_Item* objectItem =
        elm_naviframe_item_push(m_context.parent,
                                m_context.selectedOrigin.c_str(),
                                NULL,
                                NULL,
                                m_genlist,
                                NULL);
    elm_naviframe_item_pop_cb_set(objectItem, naviframeItemPopCallback, this);

    // init ctxpopup
    // 1. Delete
    CtxpopupView::CtxpopupItemDataList dataList;
    CtxpopupView::CtxpopupCallbackType deleteCallback =
        DPL::MakeDelegate(this, &PermissionView::ctxpopupDelete);
    // TODO: should use OPT type IDS
    CtxpopupView::CtxpopupItemData deleteData(WRT_SETTING_SK_DELETE,
                                              std::string(),
                                              deleteCallback);

    // 2. Reset to default
    CtxpopupView::CtxpopupCallbackType resetCallback =
        DPL::MakeDelegate(this, &PermissionView::ctxpopupReset);
    // TODO: should use OPT type IDS
    CtxpopupView::CtxpopupItemData resetData(WRT_SETTING_BODY_RESET_TO_DEFAULT,
                                              std::string(),
                                              resetCallback);
    dataList.push_back(deleteData);
    dataList.push_back(resetData);
    m_ctxpopupView->init(m_context.parent, dataList);
}

void PermissionView::loadView(void)
{
    Assert(m_genlist);
    elm_genlist_block_count_set(m_genlist, m_context.permissionData.size());

    unsigned int index = 0;
    FOREACH(it, m_context.permissionData) {
        Elm_Genlist_Item_Class* itc = elm_genlist_item_class_new();
        itc->item_style = "2text.1icon.2";
        itc->func.text_get = getTextCallback;
        itc->func.content_get = getContentCallback;
        itc->func.state_get = NULL;
        itc->func.del = delCallback;

        GenlistData* data = new GenlistData(index++, static_cast<void*>(this));
        Elm_Object_Item* objectItem =
            elm_genlist_item_append(m_genlist,
                                    itc,
                                    static_cast<void*>(data),
                                    NULL,
                                    ELM_GENLIST_ITEM_NONE,
                                    NULL,
                                    NULL);
        data->item = objectItem;
        elm_genlist_item_class_free(itc);
        if (m_context.dao->isReadOnly(*(*it))) {
            elm_object_item_disabled_set(objectItem, EINA_TRUE);
        }
        elm_genlist_item_select_mode_set(objectItem,
                                         ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
    }
}

void PermissionView::loadEmptyView(void)
{
    Assert(m_genlist);
    elm_genlist_block_count_set(m_genlist, 1);

    Elm_Genlist_Item_Class* itc = elm_genlist_item_class_new();
    itc->item_style = "default";
    itc->func.text_get = getEmptyTextCallback;
    itc->func.content_get = NULL;
    itc->func.state_get = NULL;
    itc->func.del = NULL;
    Elm_Object_Item* emptyItem =
        elm_genlist_item_append(m_genlist,
                                itc,
                                NULL,
                                NULL,
                                ELM_GENLIST_ITEM_NONE,
                                NULL,
                                NULL);
    elm_genlist_item_class_free(itc);
    elm_genlist_item_select_mode_set(emptyItem,
                                     ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
}

Eina_Bool PermissionView::naviframeItemPopCallback(void* data,
                                                   Elm_Object_Item* it)
{
    UNUSED_PARAM(it);

    Assert(data);
    PermissionView* This = static_cast<PermissionView*>(data);

    // clean-up resource
    This->m_ctxpopupView->deinit();
    This->m_ctxpopupView.reset();

    This->m_permissionDeleteView.reset();
    ea_object_event_callback_del(This->m_context.parent,
                                 EA_CALLBACK_MORE,
                                 selectedMoreCallback);
    return EINA_TRUE;
}

char* PermissionView::getEmptyTextCallback(void* data,
                                           Evas_Object* obj,
                                           const char* part)
{
    UNUSED_PARAM(data);
    UNUSED_PARAM(obj);
    UNUSED_PARAM(part);

    return strdup(WRT_SETTING_BODY_EMPTY);
}

char* PermissionView::getTextCallback(void* data,
                                      Evas_Object* obj,
                                      const char* part)
{
    UNUSED_PARAM(obj);

    // get PermissionView instance
    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    PermissionView* This = static_cast<PermissionView*>(glData->data);
    Assert(This);

    // get SecurityOriginDataPtr based current index
    SecurityOriginDataPtr originData = This->m_context.permissionData[glData->index];

    // search text based feature index
    Assert(part);
    if (ElmPart::TEXT1 == part) {
        return WrtSettingText::getFeatureText(originData->feature);
    } else if (ElmPart::TEXT2 == part) {
        return WrtSettingText::getDescriptionText(originData->feature);
    }
    return strdup("Unknown");
}

Evas_Object* PermissionView::getContentCallback(void* data,
                                                Evas_Object* obj,
                                                const char* part)
{
    UNUSED_PARAM(part);

    // get PermissionView instance
    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    PermissionView* This = static_cast<PermissionView*>(glData->data);
    Assert(This);
    Context& context = This->m_context;

    // Add check button
    Evas_Object* check = elm_check_add(obj);
    evas_object_propagate_events_set(check, EINA_FALSE);
    elm_object_style_set(check, "on&off");
    elm_object_disabled_set(check,  EINA_TRUE);

    // get result of permission
    bool result = context.getPermissionResult(glData->index);
    elm_check_state_set(check, result ? EINA_TRUE : EINA_FALSE);

    return check;
}

void PermissionView::delCallback(void* data, Evas_Object* obj)
{
    UNUSED_PARAM(obj);

    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    delete glData;
}

void PermissionView::selectedMoreCallback(void* data,
                                          Evas_Object* obj,
                                          void* eventInfo)
{
    UNUSED_PARAM(obj);
    UNUSED_PARAM(eventInfo);

    Assert(data);
    PermissionView* This = static_cast<PermissionView*>(data);
    Context& context = This->m_context;
    if (!context.permissionData.empty()) {
        This->m_ctxpopupView->show();
    }
}

void PermissionView::ctxpopupDelete(void)
{
    m_permissionDeleteView.reset();
    m_permissionDeleteView.reset(new PermissionDeleteView(m_context));
    m_permissionDeleteView->init();
}

void PermissionView::ctxpopupReset(void)
{
    m_context.removeAllPermissionData();
    updateCallback();
    m_context.cbs.originViewUpdate();
}

void PermissionView::updateCallback(void)
{
    m_context.setPermissionData();
    elm_genlist_clear(m_genlist);
    if (m_context.permissionData.empty()) {
        loadEmptyView();
    } else {
        loadView();
    }
}
} // namespace WebAppDetailSetting
